import 'package:pdf/pdf.dart';
import 'package:printing/printing.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:dio/dio.dart';
import "package:flutter/material.dart";
import 'package:verttice_ui/verttice_ui.dart';

class PdfViewerStorage extends StatefulWidget {
  final String path;
  final double? height;
  final String? diaplyName;

  const PdfViewerStorage({
    Key? key,
    required this.path,
    this.height,
    this.diaplyName,
  }) : super(key: key);

  @override
  _PdfViewerState createState() => _PdfViewerState();
}

class _PdfViewerState extends State<PdfViewerStorage> {
  late String _path;
  late Future<String?> _fetchData;
  bool _loaded = false;

  @override
  void initState() {
    _path = widget.path;
    _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);

    super.initState();
  }

  @override
  void didUpdateWidget(covariant PdfViewerStorage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_path != widget.path) {
      _path = widget.path;
      _loaded = false;
      _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<String?>(
        future: _fetchData,
        builder: (BuildContext context, AsyncSnapshot<String?> snapdata) {
          return SnapBuilder<String?>(
              height: widget.height,
              snapshot: snapdata,
              loaded: _loaded,
              erroBuilder: (dynamic error) {
                return Center(
                  child: CircleAvatar(
                    backgroundColor: Colors.red,
                    child: ThemedIcons.fileUnknow(
                      iconStyle: IconStyle(
                        color: Colors.white,
                        size: 35,
                      ),
                    ),
                  ),
                );
              },
              loadingBuilder: () => const ThemedProgress(),
              successBuilder: (String? data) {
                _loaded = true;
                return _body(
                  context,
                  data ?? '',
                );
              });
        });
  }

  Widget _body(
    BuildContext context,
    String downloadedFile,
  ) {
    return SizedBox(
      height: widget.height ?? MediaQuery.of(context).size.height / 2.5,
      child: InkWell(
        onTap: () async {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => PdfViewerFullPage(
                url: downloadedFile,
                fileName: widget.diaplyName != null
                    ? widget.diaplyName!.replaceAll(' ', '_')
                    : null,
              ),
            ),
          );
        },
        child: Container(
          decoration: const BoxDecoration(
            color: Colors.lightBlue,
            borderRadius: BorderRadius.all(
              Radius.circular(5.0),
            ),
          ),
          child: Center(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                ThemedIcons.pdfFile(
                  iconStyle: IconStyle(
                    color: Colors.white,
                  ),
                ),
                SelectableText(
                  widget.diaplyName ?? "PDF",
                  style: const TextStyle(
                    color: Colors.white,
                  ),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }
}

class PdfViewerFullPage extends StatefulWidget {
  final String url;
  final String? fileName;
  const PdfViewerFullPage({
    Key? key,
    required this.url,
    this.fileName,
  }) : super(key: key);

  @override
  _PdfViewerFullPageState createState() => _PdfViewerFullPageState();
}

class _PdfViewerFullPageState extends State<PdfViewerFullPage> {
  late String _url;
  @override
  void initState() {
    _url = widget.url;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant PdfViewerFullPage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_url != widget.url) {
      _url = widget.url;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: const ThemedAppBar(),
      body: PdfPreview(
        canChangeOrientation: false,
        pdfFileName: widget.fileName ??
            Dates.formatDataAndTime(
              miliseconds: DateTime.now().millisecondsSinceEpoch,
            ),
        build: (PdfPageFormat pdfPageFormat) async {
          final data = await Dio().get(
            _url,
            options: Options(responseType: ResponseType.bytes),
          );

          return data.data;
        },
      ),
    );
  }
}
