import "package:collection/collection.dart";
import 'package:flutter_google_places_sdk/flutter_google_places_sdk.dart';
import 'package:verttice_ui/src/base_config.dart';
import 'package:verttice_ui/src/config/config.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import "package:flutter_typeahead/flutter_typeahead.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/dialogs/full_page_dialog.dart';
import 'package:verttice_logic/verttice_logic.dart' as vl;
import 'package:verttice_ui/src/widgets/form_factory/forms_factory_index.dart';
import 'package:verttice_ui/src/widgets/goolge_maps.dart';
import 'package:verttice_ui/src/widgets/themed_app_bar.dart';
import 'package:verttice_ui/src/widgets/themed_divider.dart';

class GooglePlacesFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final Map<String, dynamic>? value;
  final Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic>? state;
  const GooglePlacesFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.state,
    this.value,
  }) : super(key: key);

  @override
  _GooglePlacesFormFactoryState createState() =>
      _GooglePlacesFormFactoryState();
}

class _GooglePlacesFormFactoryState extends State<GooglePlacesFormFactory> {
  late Map<String, dynamic>? _value;

  @override
  void initState() {
    super.initState();
    _value = widget.value;
  }

  @override
  void didUpdateWidget(GooglePlacesFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    bool eq = const MapEquality().equals(
      _value,
      widget.value,
    );

    if (!eq) {
      _value = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: Tooltip(
        message: _value?["fullAddress"] ?? "",
        child: SelectableText(
          _value?["fullAddress"] ?? "",
          maxLines: 3,
        ),
      ),
      actions: [
        PhysicalModel(
          color: Colors.grey,
          elevation: 5,
          shape: BoxShape.circle,
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: InkWell(
              child: ThemedIcons.clear(
                iconStyle: IconStyle(
                  color: Colors.white,
                ),
              ),
              onTap: () {
                widget.onChanged(widget.name, null);
                widget.state?.didChange(null);
              },
            ),
          ),
        ),
        sizedBoxW2,
      ],
      onPressed: () async {
        fullPageDialog(
          context: context,
          builder: (BuildContext context) => SearchPlace(
            address: _value != null ? vl.AddressOnMap.fromMap(_value!) : null,
            onSelect: (vl.AddressOnMap data) {
              final addressOnMapData = data.toMap();
              widget.onChanged(widget.name, addressOnMapData);
              widget.state?.didChange(addressOnMapData);
            },
          ),
        );
      },
    );
  }
}

class SearchPlace extends StatefulWidget {
  final Function(vl.AddressOnMap value) onSelect;
  final vl.AddressOnMap? address;
  const SearchPlace({
    Key? key,
    required this.onSelect,
    this.address,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _SearchPlaceState();
}

class _SearchPlaceState extends State<SearchPlace> {
  late TextEditingController _controller;

  vl.AddressOnMap? _chosenPlace;

  late final FlutterGooglePlacesSdk _places;
  final List<String> _countries = ["uk"];
  final PlaceTypeFilter _placeTypeFilter = PlaceTypeFilter.ADDRESS;
  final List<PlaceField> _placeFields = [
    PlaceField.Address,
    PlaceField.AddressComponents,
    PlaceField.Id,
    PlaceField.Location,
    PlaceField.Name,
    PlaceField.PhoneNumber,
    PlaceField.PlusCode,
    PlaceField.Types,
    PlaceField.UserRatingsTotal,
    PlaceField.UTCOffset,
    PlaceField.Viewport,
    PlaceField.WebsiteUri,
  ];

  @override
  void initState() {
    super.initState();

    _places = FlutterGooglePlacesSdk(VertticeUIConfig.instance.firebaseApiKey);

    _chosenPlace = widget.address;

    _controller = TextEditingController.fromValue(
      TextEditingValue(
        text: _chosenPlace?.address ?? '',
      ),
    );
  }

  @override
  void didUpdateWidget(SearchPlace oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_chosenPlace?.latLng?.lat != widget.address?.latLng?.lat ||
        _chosenPlace?.latLng?.lng != widget.address?.latLng?.lng) {
      _chosenPlace = widget.address;

      _controller = TextEditingController.fromValue(
        TextEditingValue(
          text: _chosenPlace?.address ?? '',
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        actions: [
          if (_chosenPlace != null)
            Tooltip(
              message: "Select place",
              child: IconButton(
                onPressed: () {
                  widget.onSelect(_chosenPlace ?? const vl.AddressOnMap());

                  Navigator.of(context).pop();
                },
                icon: ThemedIcons.check(),
              ),
            ),
        ],
        title: "Find a place",
      ),
      body: Padding(
        padding: const EdgeInsets.all(30),
        child: LayoutBuilder(
          builder: (context, constraints) {
            return Flex(
              direction: constraints.maxWidth < mobileWidth
                  ? Axis.vertical
                  : Axis.horizontal,
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                SizedBox(
                  width: (constraints.maxWidth / 2) - 10,
                  child: Column(
                    children: [
                      SizedBox(
                        height: 100,
                        child: Center(
                          child: TextFormFactory(
                            value: _chosenPlace?.addressPrefix ?? '',
                            displayName:
                                'Property prefix (e.g. flat, property name)',
                            name: 'placePrefix',
                            onChanged: (String name, dynamic data) {
                              final currrntPlace =
                                  _chosenPlace ?? const vl.AddressOnMap();
                              setState(() {
                                _chosenPlace = currrntPlace.copyWith(
                                  fullAddress:
                                      "$data${data == '' ? '' : ' '}${currrntPlace.address ?? ""}",
                                  addressPrefix: data,
                                );
                              });
                            },
                            showStandartBorder: false,
                          ),
                        ),
                      ),
                      TypeAheadField(
                        textFieldConfiguration: TextFieldConfiguration(
                          style: DefaultTextStyle.of(context).style.copyWith(
                                fontStyle: FontStyle.italic,
                              ),
                          controller: _controller,
                          decoration: const InputDecoration(
                            border: OutlineInputBorder(),
                            hintText: "Search a place",
                          ),
                        ),
                        suggestionsCallback: (pattern) async {
                          List<AutocompletePrediction> predictions = [];
                          if (pattern == "") {
                            return predictions;
                          }
                          final result =
                              await _places.findAutocompletePredictions(
                            pattern,
                            countries: _countries,
                            placeTypeFilter: _placeTypeFilter,
                            newSessionToken: false,
                          );
                          return result.predictions;
                        },
                        itemBuilder:
                            (context, AutocompletePrediction suggestion) {
                          return ListTile(
                            leading: ThemedIcons.pin(),
                            title: Text(suggestion.fullText),
                            subtitle: Text(
                              suggestion.primaryText +
                                  " - " +
                                  suggestion.secondaryText,
                            ),
                          );
                        },
                        onSuggestionSelected:
                            (AutocompletePrediction suggestion) async {
                          final result = await _places.fetchPlace(
                            suggestion.placeId,
                            fields: _placeFields,
                          );
                          setState(() {
                            _chosenPlace = vl.AddressOnMap(
                              address: result.place?.addressComponents
                                  ?.map((e) => e.name)
                                  .toList()
                                  .join(" "),
                              latLng: result.place?.latLng != null
                                  ? vl.LatLng.fromMap(
                                      result.place!.latLng!.toMap())
                                  : null,
                              name: result.place?.name,
                              utcOffsetMinutes: result.place?.utcOffsetMinutes,
                              viewport: result.place?.viewport != null
                                  ? vl.LatLngBounds.fromMap(
                                      result.place!.viewport!.toMap())
                                  : null,
                              websiteUri: result.place?.websiteUri?.toString(),
                            );
                            _controller = TextEditingController.fromValue(
                              TextEditingValue(
                                text: _chosenPlace?.address ?? '',
                              ),
                            );
                          });
                        },
                      )
                    ],
                  ),
                ),
                ThemedDivider(width: 2, height: constraints.maxHeight),
                if (_chosenPlace?.latLng?.lat != null &&
                    _chosenPlace?.latLng?.lng != null)
                  SizedBox(
                    width: (constraints.maxWidth / 2) - 10,
                    child: ListView(
                      controller: ScrollController(),
                      children: [
                        SizedBox(
                          height: constraints.maxHeight == double.infinity
                              ? 600
                              : constraints.maxHeight,
                          width: (constraints.maxWidth / 2) - 10,
                          child: MapCoordinatesPage(
                            lat: _chosenPlace!.latLng!.lat,
                            lng: _chosenPlace!.latLng!.lng,
                            placeName:
                                "${_chosenPlace?.addressPrefix ?? ""}${_chosenPlace?.addressPrefix == null || _chosenPlace?.addressPrefix == '' ? '' : ' '}${_chosenPlace?.address ?? ""}",
                          ),
                        )
                      ],
                    ),
                  ),
              ],
            );
          },
        ),
      ),
    );
  }
}
