import "package:collection/collection.dart";
import "package:flutter/material.dart";
import 'package:verttice_logic/verttice_logic.dart';
import "dart:math";
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/services.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';

class NumberRangeFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final Map<String, double?>? value;
  final FormFieldState<dynamic>? state;
  const NumberRangeFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.value,
    this.state,
  }) : super(key: key);

  @override
  _NumberRangeFormState createState() => _NumberRangeFormState();
}

class _NumberRangeFormState extends State<NumberRangeFormFactory> {
  var _value = <String, double?>{};

  @override
  void initState() {
    _value = DataClassUtils.dynamicToMapStringDynamic(widget.value)
        .cast<String, double?>();
    super.initState();
  }

  @override
  void didUpdateWidget(covariant NumberRangeFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const MapEquality().equals(_value, widget.value)) {
      _value = DataClassUtils.dynamicToMapStringDynamic(widget.value)
          .cast<String, double?>();
    }
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraints) {
      return BorderLabel(
        label: widget.displayName,
        child: Row(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SizedBox(
                width: (constraints.maxWidth / 2) - 12.5,
                child: NumberRangeInput(
                    displayName: "Bigger Than",
                    name: "smallNumber",
                    value: _value["smallNumber"],
                    onChanged: (String name, dynamic val) {
                      setState(() {
                        _value = {..._value, name: val as double};
                      });
                      final newVal = {..._value, name: val as double};
                      widget.onChanged(widget.name, newVal);
                      widget.state?.didChange(newVal);
                    }),
              ),
              SizedBox(
                width: (constraints.maxWidth / 2) - 12.5,
                child: NumberRangeInput(
                    displayName: "Less Than",
                    name: "bigNumber",
                    value: _value["bigNumber"],
                    onChanged: (String name, dynamic val) {
                      setState(() {
                        _value = {..._value, name: val as double};
                      });
                      final newVal = {..._value, name: val as double};
                      widget.onChanged(widget.name, newVal);
                      widget.state?.didChange(newVal);
                    }),
              ),
            ]),
      );
    });
  }
}

class NumberRangeInput extends StatefulWidget {
  final String displayName;
  final String name;
  final dynamic value;

  final Function(String name, dynamic value) onChanged;

  const NumberRangeInput({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.value,
  }) : super(key: key);
  @override
  _TextFormState createState() => _TextFormState();
}

class _TextFormState extends State<NumberRangeInput> {
  TextEditingController? _controller;
  String _value = "";
  int _baseOffset = 0;
  late FocusNode _focusNode;
  @override
  void initState() {
    super.initState();

    _value = DataClassUtils.dynamicToDouble(widget.value).toString();
    _controller = TextEditingController(
      text: _value,
    );

    _controller!.selection = TextSelection.fromPosition(
      TextPosition(
        offset: min(
          _baseOffset,
          _controller != null ? _controller!.text.length : 0,
        ),
      ),
    );

    _controller!.addListener(_listen);
    _focusNode = FocusNode();
  }

  @override
  void dispose() {
    _focusNode.dispose();
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant NumberRangeInput oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToDouble(widget.value).toString();

      _controller = TextEditingController(
        text: _value,
      );
      _controller!.selection = TextSelection.fromPosition(
        TextPosition(
          offset: min(
            _baseOffset,
            _controller != null ? _controller!.text.length : 0,
          ),
        ),
      );
      _controller!.addListener(_listen);
    }
  }

  void _listen() {
    setState(() {
      _baseOffset = _controller!.selection.baseOffset;
    });
  }

  @override
  Widget build(BuildContext context) {
    TextInputType inputType = TextInputType.number;

    return LayoutBuilder(builder: (context, constraints) {
      return SizedBox(
        width: constraints.maxWidth,
        child: MouseRegion(
          onEnter: (_) {
            setState(() {
              _focusNode.requestFocus();
            });
          },
          onExit: (_) {
            setState(() {
              _focusNode.unfocus();
            });
          },
          child: TextFormField(
            focusNode: _focusNode,
            cursorColor: Colors.amberAccent,
            onChanged: (val) {
              setState(() {
                _value = val;
              });
              final newVal = val != "" ? double.parse(val) : null;
              widget.onChanged(widget.name, newVal);
            },
            controller: _controller,
            keyboardType: inputType,
            inputFormatters: <TextInputFormatter>[
              FilteringTextInputFormatter.allow(RegExp(r"(^\-?\d*\.?\d*)")),
            ],
            textDirection: TextDirection.ltr,
            autofocus: false,
            decoration: InputDecoration(
              label: Text(widget.displayName),
              suffixIcon: IconButton(
                onPressed: () {
                  _controller!.clear();
                  widget.onChanged(widget.name, "");
                },
                icon: ThemedIcons.clear(
                  iconStyle: IconStyle(
                    color: Colors.redAccent,
                  ),
                ),
              ),
            ),
          ),
        ),
      );
    });
  }
}
