import 'dart:convert';
import "package:flutter/material.dart";
import 'package:validators/validators.dart';
import 'package:verttice_ui/verttice_ui.dart';

class RichTextFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final dynamic value;
  final Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic>? state;

  const RichTextFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.state,
    this.value,
  }) : super(key: key);
  @override
  _TextFormState createState() => _TextFormState();
}

class _TextFormState extends State<RichTextFormFactory> {
  String _value = "";

  @override
  void initState() {
    super.initState();
    _value = widget.value;
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant RichTextFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      onPressed: () async {
        await fullPageDialog(
            context: context,
            builder: (BuildContext context) => Builder(builder: (context) {
                  return Scaffold(
                    appBar: const ThemedAppBar(),
                    body: ThemedEditor(
                      jsonContect: isJSON(_value) ? json.decode(_value) : null,
                      fileContentCallback: ({
                        required List<dynamic> jsonContect,
                      }) async {
                        final text = json.encode(jsonContect);
                        setState(() {
                          _value = text;
                        });
                        widget.onChanged(widget.name, text);
                        widget.state?.didChange(text);
                        Navigator.pop(context);
                      },
                    ),
                  );
                }));
      },
      child: LayoutBuilder(builder: (context, constraints) {
        return SizedBox(
          width: constraints.maxWidth,
          height: constraints.maxHeight,
          child: SingleChildScrollView(
            child: ThemedEditor(
              jsonContect: isJSON(_value) ? json.decode(_value) : null,
              readonly: true,
            ),
          ),
        );
      }),
    );
  }
}
