import 'package:flutter/material.dart';
import 'package:flutter_quill/flutter_quill.dart' hide Text;
import 'package:verttice_ui/verttice_ui.dart';
import 'web_video_player.dart';
import 'web_youtube_player.dart';

Widget defaultEmbedBuilderWeb(
  BuildContext context,
  QuillController controller,
  Embed node,
  bool readOnly,
) {
  switch (node.value.type) {
    case BlockEmbed.imageType:
      final imageUrl = node.value.data;
      return ConstrainedBox(
        constraints: const BoxConstraints(maxHeight: 350),
        child: InkWell(
          onTap: () async {
            Navigator.push(context,
                MaterialPageRoute(builder: (BuildContext context) {
              return ImageViewerFullPage(
                imageID: imageUrl,
                url: imageUrl,
              );
            }));
          },
          child: Image.network(
            imageUrl,
            fit: BoxFit.contain,
          ),
        ),
      );
    case BlockEmbed.videoType:
      final videoUrl = node.value.data;
      if (videoUrl.contains('youtube.com') || videoUrl.contains('youtu.be')) {
        return WebYoutubeVideoApp(
          videoUrl: videoUrl,
          context: context,
          readOnly: readOnly,
        );
      }

      return WebVideoApp(
        videoUrl: videoUrl,
        context: context,
        readOnly: readOnly,
      );
    default:
      throw UnimplementedError(
        'Embeddable type "${node.value.type}" is not supported by default '
        'embed builder of QuillEditor. You must pass your own builder function '
        'to embedBuilder property of QuillEditor or QuillField widgets.',
      );
  }
}

String replaceStyleString(String s, double width, double height) {
  final result = <String, String>{};
  final pairs = s.split(';');
  for (final pair in pairs) {
    final _index = pair.indexOf(':');
    if (_index < 0) {
      continue;
    }
    final _key = pair.substring(0, _index).trim();
    result[_key] = pair.substring(_index + 1).trim();
  }

  result[Attribute.mobileWidth] = width.toString();
  result[Attribute.mobileHeight] = height.toString();
  final sb = StringBuffer();
  for (final pair in result.entries) {
    sb
      ..write(pair.key)
      ..write(': ')
      ..write(pair.value)
      ..write('; ');
  }
  return sb.toString();
}

Alignment getAlignment(String? s) {
  const _defaultAlignment = Alignment.center;
  if (s == null) {
    return _defaultAlignment;
  }

  final _index = [
    'topLeft',
    'topCenter',
    'topRight',
    'centerLeft',
    'center',
    'centerRight',
    'bottomLeft',
    'bottomCenter',
    'bottomRight'
  ].indexOf(s);
  if (_index < 0) {
    return _defaultAlignment;
  }

  return [
    Alignment.topLeft,
    Alignment.topCenter,
    Alignment.topRight,
    Alignment.centerLeft,
    Alignment.center,
    Alignment.centerRight,
    Alignment.bottomLeft,
    Alignment.bottomCenter,
    Alignment.bottomRight
  ][_index];
}

Map<String, String> parseKeyValuePairs(String s, Set<String> targetKeys) {
  final result = <String, String>{};
  final pairs = s.split(';');
  for (final pair in pairs) {
    final _index = pair.indexOf(':');
    if (_index < 0) {
      continue;
    }
    final _key = pair.substring(0, _index).trim();
    if (targetKeys.contains(_key)) {
      result[_key] = pair.substring(_index + 1).trim();
    }
  }

  return result;
}

class EditorResponsiveWidget extends StatelessWidget {
  const EditorResponsiveWidget({
    required this.largeScreen,
    this.mediumScreen,
    this.smallScreen,
    Key? key,
  }) : super(key: key);

  final Widget largeScreen;
  final Widget? mediumScreen;
  final Widget? smallScreen;

  static bool isSmallScreen(BuildContext context) {
    return MediaQuery.of(context).size.width < 800;
  }

  static bool isLargeScreen(BuildContext context) {
    return MediaQuery.of(context).size.width > 1200;
  }

  static bool isMediumScreen(BuildContext context) {
    return MediaQuery.of(context).size.width >= 800 &&
        MediaQuery.of(context).size.width <= 1200;
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, constraints) {
        if (constraints.maxWidth > 1200) {
          return largeScreen;
        } else if (constraints.maxWidth <= 1200 &&
            constraints.maxWidth >= 800) {
          return mediumScreen ?? largeScreen;
        } else {
          return smallScreen ?? largeScreen;
        }
      },
    );
  }
}
